// Communication Functions in c for PC

#include "base.h"

//*************************************************************
//               Receive and Send: Low Level
//*************************************************************

#define SERIC_PC_COM1     0x3f8
#define SERIC_PC_COM2     0x2f8

// Set up the serial port

int seric_InitCOM (int Port, long Baud) {
   seric_outp(Port+3, 0x80);
   seric_outp(Port, 115200/Baud);
   seric_outp(Port+1, 0x00);
   seric_outp(Port+3, 0x03);
   seric_outp(Port+6, 0x00);
}

// Send a byte

void seric_Out (int Port, char ch) {
   char check;
   do {check=0x20&seric_inp(Port+5);} while(!check);
   seric_outp(Port, ch);
}

// Read a byte into *ch and return true, if received

int seric_In (int Port, char *ch) {
   char check=seric_inp(Port+5);
   if( 0x1&check ) {
      *ch=seric_inp(Port);
      return 1;}
   else return 0;
}

// Send n bytes from Buffer

void seric_Send (int Port, char *Buffer, int n) {
   int i;
   for(i=0; i<n; i++) seric_Out(Port, Buffer[i]);
}


//*************************************************************
//                   Send: Middle Level
//*************************************************************

// Send the size-byte and the content except the null, of a null-terminated string

void seric_SendString1 (int Port, char *String) {
   unsigned char n=strlen(String);
   seric_Out(Port, n);
   seric_Send(Port, String, n);
}

// Send coordinates of a point or a rectangle

#define seric_SendPoint(Port, pt) seric_Send(Port, (char*)pt, sizeof(SERICOND_Point))

#define seric_SendRect(Port, rc) seric_Send(Port, (char*)rc, sizeof(SERICOND_Rect))

// Send null rectangle coordinates

void seric_SendRectNull (int Port, SERICOND_Rect *rc) {
   SERICOND_Rect Rect={0, 0, 0, 0};
   if(rc==0) {
      rc=&Rect;}
   seric_SendRect(Port, rc);
}

// Send coordinates of a rectangle, and a text-drawing-format byte

void seric_SendRectDT (int Port, SERICOND_Rect *rc, unsigned char fDT) {
   seric_SendRectNull(Port, rc);
   seric_Out(Port, (char)fDT);
}


//*************************************************************
//                 Send: High Level
//*************************************************************

// Turn into Function Mode

void seric_FunctionMode (int Port) {
   seric_Out(Port, 0);
   seric_Out(Port, 0);
   seric_Out(Port, 0);
}

// "Printf" String or Variable

void seric_PrintStr (int Port, char *Str) {
   seric_Out(Port, SERIC_FTB_str);
   seric_SendString1(Port, Str);
}

void seric_Print1u (int Port, char *Format, unsigned char Var) {
   seric_Out(Port, SERIC_FTB_1u);
   seric_SendString1(Port, Format);
   seric_Out(Port, (char)Var);
}

void seric_Print1 (int Port, char *Format, char Var) {    
   seric_Out(Port, SERIC_FTB_1);
   seric_SendString1(Port, Format);
   seric_Out(Port, Var);
}

void seric_Print2u (int Port, char *Format, unsigned short Var) {
   seric_Out(Port, SERIC_FTB_2u);
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
}

void seric_Print2 (int Port, char *Format, short Var) {
   seric_Out(Port, SERIC_FTB_2);
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
}

void seric_Print4 (int Port, char *Format, long Var) {   
   seric_Out(Port, SERIC_FTB_4);
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
}

void seric_Print8f (int Port, char *Format, double Var) { 
   seric_Out(Port, SERIC_FTB_8f);
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
}

// Paint Tools

void seric_NewBrush (int Port, char iTool,
                        unsigned long clr) {
   seric_Out(Port, SERIC_FTB_NewBrush);
   seric_Out(Port, iTool);
   seric_Send(Port, (char*)&clr, sizeof(clr));
}

void seric_NewPen (int Port, char iTool,
                      unsigned long clr, short width) {
   seric_Out(Port, SERIC_FTB_NewPen);
   seric_Out(Port, iTool);
   seric_Send(Port, (char*)&clr, sizeof(clr));
   seric_Send(Port, (char*)&width, sizeof(width));
}

void seric_NewFont (int Port, char iTool,
                       char *name, short width, short height, short weight, unsigned char flag) {
   seric_Out(Port, SERIC_FTB_NewFont);
   seric_Out(Port, iTool);
   seric_Send(Port, (char*)&width, sizeof(width));
   seric_Send(Port, (char*)&height, sizeof(height));
   seric_Send(Port, (char*)&weight, sizeof(weight));
   seric_Out(Port, (char)flag);
}

// Canvas

void seric_NewCanvas (int Port, short width, short height) {
   seric_Out(Port, SERIC_FTB_NewCanvas);
   seric_Send(Port, (char*)&width, sizeof(width));
   seric_Send(Port, (char*)&height, sizeof(height));
}

void seric_ShowCanvas (int Port, SERICOND_Rect *rc, char iBr) {
   seric_Out(Port, SERIC_FTB_ShowCanvas);
   seric_SendRectNull(Port, rc);
   seric_Out(Port, iBr);
}

void seric_SelectPaintTool (int Port, char iTool) {
   seric_Out(Port, SERIC_FTB_SelectPaintTool);
   seric_Out(Port, iTool);
}

void seric_SetTextColor (int Port, unsigned long clr) {
   seric_Out(Port, SERIC_FTB_SetTextColor);
   seric_Send(Port, (char*)&clr, sizeof(clr));
}

// Draw String or Variable

void seric_DrawStr (int Port, char *Str,
                    SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_Gstr);   
   seric_SendString1(Port, Str);
   seric_SendRectDT(Port, rc, fDT);
}

void seric_Draw1u (int Port, char *Format, unsigned char Var,
                   SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_G1u);                        
   seric_SendString1(Port, Format);
   seric_Out(Port, (char)Var);
   seric_SendRectDT(Port, rc, fDT);
}

void seric_Draw1 (int Port, char *Format, char Var,
                   SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_G1);                         
   seric_SendString1(Port, Format);
   seric_Out(Port, Var);
   seric_SendRectDT(Port, rc, fDT);
}

void seric_Draw2u (int Port, char *Format, unsigned short Var,
                   SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_G2u);                        
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
   seric_SendRectDT(Port, rc, fDT);
}

void seric_Draw2 (int Port, char *Format, short Var,
                   SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_G2);                         
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
   seric_SendRectDT(Port, rc, fDT);
}

void seric_Draw4 (int Port, char *Format, long Var,
                   SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_G4);
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
   seric_SendRectDT(Port, rc, fDT);
}

void seric_Draw8f (int Port, char *Format, double Var,
                   SERICOND_Rect *rc, unsigned char fDT) {
   seric_Out(Port, SERIC_FTB_G8f);                        
   seric_SendString1(Port, Format);
   seric_Send(Port, (char*)&Var, sizeof(Var));
   seric_SendRectDT(Port, rc, fDT);
}

// Brush Drawing

void seric_Solid (int Port, SERICOND_Rect *rc, char iBr) {
   seric_Out(Port, SERIC_FTB_Solid);
   seric_SendRectNull(Port, rc);
   seric_Out(Port, iBr);
}

void seric_Frame (int Port, SERICOND_Rect *rc, char iBr) {
   seric_Out(Port, SERIC_FTB_Frame);
   seric_SendRectNull(Port, rc);
   seric_Out(Port, iBr);
}

// Pen Drawing

void seric_Line (int Port, SERICOND_Point *pt1, SERICOND_Point *pt2) {
   seric_Out(Port, SERIC_FTB_Line);
   seric_SendPoint(Port, pt1);
   seric_SendPoint(Port, pt2);
}

void seric_Arc (int Port, SERICOND_Rect *rc, SERICOND_Point *pt1, SERICOND_Point *pt2) {
   seric_Out(Port, SERIC_FTB_Arc);
   seric_SendRectNull(Port, rc);
   seric_SendPoint(Port, pt1);
   seric_SendPoint(Port, pt2);
}

// Pen+Brush Drawing

void seric_Rectangle (int Port, SERICOND_Rect *rc) {
   seric_Out(Port, SERIC_FTB_Rectangle);
   seric_SendRectNull(Port, rc);
}

void seric_Ellipse (int Port, SERICOND_Rect *rc) {
   seric_Out(Port, SERIC_FTB_Ellipse);
   seric_SendRectNull(Port, rc);
}

void seric_rRectangle (int Port, SERICOND_Rect *rc, short wRound, short hRound) {
   seric_Out(Port, SERIC_FTB_rRectangle);
   seric_SendRectNull(Port, rc);
   seric_Send(Port, (char*)&wRound, sizeof(wRound));
   seric_Send(Port, (char*)&hRound, sizeof(hRound));
}

void seric_Pie (int Port, SERICOND_Rect *rc, SERICOND_Point *pt1, SERICOND_Point *pt2) {
   seric_Out(Port, SERIC_FTB_Pie);
   seric_SendRectNull(Port, rc);
   seric_SendPoint(Port, pt1);
   seric_SendPoint(Port, pt2);
}

void seric_Chord (int Port, SERICOND_Rect *rc, SERICOND_Point *pt1, SERICOND_Point *pt2) {
   seric_Out(Port, SERIC_FTB_Chord);
   seric_SendRectNull(Port, rc);
   seric_SendPoint(Port, pt1);
   seric_SendPoint(Port, pt2);
}

// Image Operation

void seric_Invert (int Port, SERICOND_Rect *rc) {
   seric_Out(Port, SERIC_FTB_Invert);
   seric_SendRectNull(Port, rc);
}

// Mouse

void seric_EnableClicks (int Port, char LeftClick, char RightClick) {
   seric_Out(Port, SERIC_FTB_EnableClicks);
   seric_Out(Port, LeftClick);
   seric_Out(Port, RightClick);
}

